#include "pinDefinitions.h"
#include "si5351_lite.h"
#include "encoder.h"


/* "quarzersatz"
   Version: ..\Si5351_vfo_terminal_xxx.ino
   vom Aug 24 2018
   NNNNNNNNa : Frequenzeingabe NNNNNNNN in Hertz
          xd : Drive ( 0= 2mA, 1= 4mA, 2= 6ma, 3= 8mA)
           ? : Display Version und aktuelle Daten

   Si-Modul Quarz Einstellen siehe "si5351_lite.h"
   #define XTAL 27005000
*/
#define FREQUENZ 63.950

//++++++++++++++++++++++++++++++++++++++++++++
#include <SoftwareSerial.h>
SoftwareSerial mySerial(RX, TX);
/* Software Serial benutzt beliebige pins
    fuer rx und tx
    und ist kürzer als hardware-"Serial"
*/

#include "si5351_lite.h"
/* lite version ist wesentlich kuerzer als die
    offizielle si5351 version von adafruit
*/
si5351_lite   si5351;

#define STARTFREQUENZ FREQUENZ*1e6
#define STARTDrive 0 /* 2mA */
unsigned long serial_input_number;
const bool OK = true;

#include <EEPROM.h>
const int EEPROM_STARTADRESSE = 0;
const unsigned long MAGIC_CODE = 0x12345678L; // magic code
struct eeprom_datensatz {
  unsigned long frequenz;
  unsigned char drive;
  unsigned long identifier;
};
eeprom_datensatz eeData;

Encoder encoder(5);// alle 5 millisec in encoder.update() testen
//unsigned long myFrequency;

////////////////////////////////////////////////////////////////
void setup()
{ mySerial.begin(9600);
  displayVersion();
  si5351.init();
  EEPROM.get(EEPROM_STARTADRESSE, eeData);
  if ( eeData.identifier == MAGIC_CODE ) // magic code ok
  { //frequenz aus eeprom ok -> einstellen
  }
  else
  { eeData.frequenz =  STARTFREQUENZ;  // standartwerte einstellen
    eeData.drive = STARTDrive;
    eeData.identifier = MAGIC_CODE;
    EEPROM.put(EEPROM_STARTADRESSE, eeData);
  }
  setNewData(&eeData); // eeprom daten ins si5351 übernehmen
  display();
  encoder.modify( &eeData.frequenz, 100L  ); // Frequency 100 rauf oder runter
  pinMode(PIN_SW, INPUT_PULLUP);
}

////////////////////////////////////////////////////////////////
void loop()
{ encoder.update();       // periodisch!!
  if (encoder.moved())    // ereignis: hat sich bewegt!
  { mySerial.println(eeData.frequenz);
    setNewData(&eeData);  //neue frequenz
  }
  if ( digitalRead(PIN_SW) == LOW )         //knopf gedrückt ? -->Speichern
  { setNewData(&eeData);                    //frequenz neu einstellen
    EEPROM.put(EEPROM_STARTADRESSE, eeData);//write eeprom
    display();
    delay(1000); //pause, entprellen
  }
}

////////////////////////////////////////////////////////////////
static void displayVersion()
{ mySerial.print(F("Version: .."));
  char *s;
  s = strrchr(__FILE__, '\\');
  mySerial.println(s);
  mySerial.println(F("vom " __DATE__ " " __TIME__));
}

////////////////////////////////////////////////////////////////
static void display()
{ mySerial.print(F( "frequenz= "));
  mySerial.print( eeData.frequenz );
  mySerial.println(F( " Hertz"));
  mySerial.print(F( "Drive= "));
  mySerial.print( ( eeData.drive + 1) * 2 );
  mySerial.println(F( " mA"));
}


////////////////////////////////////////////////////////////////
void setNewData(eeprom_datensatz *ee)
{ si5351.setDrive( 0, ee->drive);
  si5351.setfreq ( 0, ee->frequenz);
}



